@@ -128,8 +128,7 @@ static const char* primitiveTypeLabel(char typeChar)
 /*
  * Converts a type descriptor to human-readable "dotted" form.  For
  * example, "Ljava/lang/String;" becomes "java.lang.String", and
- * "[I" becomes "int[]".  Also converts '$' to '.', which means this
- * form can't be converted back to a descriptor.
+ * "[I" becomes "int[]".
  */
 static char* descriptorToDot(const char* str)
 {
@@ -166,7 +165,7 @@ static char* descriptorToDot(const char* str)
     int i;
     for (i = 0; i < targetLen; i++) {
         char ch = str[offset + i];
-        newStr[i] = (ch == '/' || ch == '$') ? '.' : ch;
+        newStr[i] = (ch == '/') ? '.' : ch;
     }
 
     /* add the appropriate number of brackets for arrays */
@@ -181,16 +180,14 @@ static char* descriptorToDot(const char* str)
 }
 
 /*
- * Converts the class name portion of a type descriptor to human-readable
- * "dotted" form.
+ * Retrieves the class name portion of a type descriptor.
  *
  * Returns a newly-allocated string.
  */
-static char* descriptorClassToDot(const char* str)
+static char* descriptorClassToName(const char* str)
 {
     const char* lastSlash;
     char* newStr;
-    char* cp;
 
     /* reduce to just the class name, trimming trailing ';' */
     lastSlash = strrchr(str, '/');
@@ -201,10 +198,6 @@ static char* descriptorClassToDot(const char* str)
 
     newStr = strdup(lastSlash);
     newStr[strlen(lastSlash)-1] = '\0';
-    for (cp = newStr; *cp != '\0'; cp++) {
-        if (*cp == '$')
-            *cp = '.';
-    }
 
     return newStr;
 }
@@ -738,7 +731,8 @@ bool getProtoInfo(DexFile* pDexFile, u4 protoIdx, ProtoInfo* pProtoInfo)
             return false;
         }
         const char* param = dexStringByTypeIdx(pDexFile, paramTypes->list[i].typeIdx);
-        size_t newUsed = bufUsed + strlen(param);
+        size_t paramLen = strlen(param);
+        size_t newUsed = bufUsed + paramLen;
         if (newUsed > bufSize) {
             char* newBuf = (char*)realloc(buf, newUsed);
             if (newBuf == NULL) {
@@ -748,7 +742,7 @@ bool getProtoInfo(DexFile* pDexFile, u4 protoIdx, ProtoInfo* pProtoInfo)
             buf = newBuf;
             bufSize = newUsed;
         }
-        strncat(buf + bufUsed - 1, param, bufSize - (bufUsed - 1));
+        memcpy(buf + bufUsed - 1, param, paramLen + 1);
         bufUsed = newUsed;
     }
 
@@ -915,9 +909,27 @@ static char* indexString(DexFile* pDexFile, const DecodedInstruction* pDecInsn,
             free(protoInfo.parameterTypes);
         }
         break;
-    case kCallSiteRef:
+    case kIndexCallSiteRef:
         outSize = snprintf(buf, bufSize, "call_site@%0*x", width, index);
         break;
+    case kIndexMethodHandleRef:
+        outSize = snprintf(buf, bufSize, "methodhandle@%0*x", width, index);
+        break;
+    case kIndexProtoRef:
+        {
+            ProtoInfo protoInfo;
+            if (getProtoInfo(pDexFile, index, &protoInfo)) {
+                outSize = snprintf(buf, bufSize, "(%s)%s // proto@%0*x",
+                                   protoInfo.parameterTypes, protoInfo.returnType,
+                                   width, index);
+
+            } else {
+                outSize = snprintf(buf, bufSize, "<proto?> // proto@%0*x",
+                                   width, secondaryIndex);
+            }
+            free(protoInfo.parameterTypes);
+        }
+        break;
     default:
         outSize = snprintf(buf, bufSize, "<?>");
         break;
@@ -1306,7 +1318,7 @@ void dumpMethod(DexFile* pDexFile, const DexMethod* pDexMethod, int i)
         if (constructor) {
             char* tmp;
 
-            tmp = descriptorClassToDot(backDescriptor);
+            tmp = descriptorClassToName(backDescriptor);
             printf("<constructor name=\"%s\"\n", tmp);
             free(tmp);
 
@@ -1571,7 +1583,7 @@ void dumpClass(DexFile* pDexFile, int idx, char** pLastPackage)
     } else {
         char* tmp;
 
-        tmp = descriptorClassToDot(classDescriptor);
+        tmp = descriptorClassToName(classDescriptor);
         printf("<class name=\"%s\"\n", tmp);
         free(tmp);
 
@@ -1644,15 +1656,6 @@ bail:
 }
 
 
-/*
- * Advance "ptr" to ensure 32-bit alignment.
- */
-static inline const u1* align32(const u1* ptr)
-{
-    return (u1*) (((uintptr_t) ptr + 3) & ~0x03);
-}
-
-
 /*
  * Dump a map in the "differential" format.
  *
@@ -1978,7 +1981,6 @@ static u8 readUnsignedLittleEndian(const u1** pData, u4 size, bool fillOnRight =
         result = (result >> 8) | (((u8)*data++) << 56);
     }
 
-    u8 oldResult = result;
     if (!fillOnRight) {
         result >>= (8u - size) * 8;
     }
