@@ -25,7 +25,6 @@
 #include "DexUtf.h"
 #include "Leb128.h"
 
-#include <safe_iop.h>
 #include <zlib.h>
 
 #include <stdlib.h>
@@ -140,7 +139,8 @@ static inline bool checkPtrRange(const CheckState* state,
 #define CHECK_LIST_SIZE(_ptr, _count, _elemSize) {                          \
         const u1* _start = (const u1*) (_ptr);                              \
         const u1* _end = _start + ((_count) * (_elemSize));                 \
-        if (!safe_mul(NULL, (_count), (_elemSize)) ||                       \
+        u4 _dummy;                                                          \
+        if (__builtin_mul_overflow((_count), (_elemSize), &_dummy) ||       \
             !checkPtrRange(state, _start, _end, #_ptr)) {                   \
             return 0;                                                       \
         }                                                                   \
@@ -617,7 +617,7 @@ static bool shortyDescMatch(char shorty, const char* descriptor, bool
                 ALOGE("Invalid use of void");
                 return false;
             }
-            // Fall through.
+            FALLTHROUGH_INTENDED;
         }
         case 'B':
         case 'C':
@@ -2464,19 +2464,6 @@ static void* intraVerifyAnnotationItem(const CheckState* state, void* ptr) {
     return (void*) verifyEncodedAnnotation(state, data, false);
 }
 
-/* Perform cross-item verification on annotation_item. */
-static void* crossVerifyAnnotationItem(const CheckState* state, void* ptr) {
-    const u1* data = (const u1*) ptr;
-
-    // Skip the visibility byte.
-    data++;
-
-    return (void*) verifyEncodedAnnotation(state, data, true);
-}
-
-
-
-
 /*
  * Function to visit an individual top-level item type.
  */
@@ -2921,7 +2908,8 @@ bool dexHasValidMagic(const DexHeader* pHeader)
     if ((memcmp(version, DEX_MAGIC_VERS, 4) != 0) &&
         (memcmp(version, DEX_MAGIC_VERS_API_13, 4) != 0) &&
         (memcmp(version, DEX_MAGIC_VERS_37, 4) != 0) &&
-        (memcmp(version, DEX_MAGIC_VERS_38, 4) != 0)) {
+        (memcmp(version, DEX_MAGIC_VERS_38, 4) != 0) &&
+        (memcmp(version, DEX_MAGIC_VERS_39, 4) != 0)) {
         /*
          * Magic was correct, but this is an unsupported older or
          * newer format variant.
