@@ -18,12 +18,10 @@ package com.android.dx.dex.code;
 
 import com.android.dx.rop.code.RegisterSpec;
 import com.android.dx.rop.code.RegisterSpecList;
-import com.android.dx.rop.cst.Constant;
 import com.android.dx.rop.cst.CstInteger;
 import com.android.dx.rop.cst.CstKnownNull;
 import com.android.dx.rop.cst.CstLiteral64;
 import com.android.dx.rop.cst.CstLiteralBits;
-import com.android.dx.rop.cst.CstString;
 import com.android.dx.util.AnnotatedOutput;
 import com.android.dx.util.Hex;
 import java.util.BitSet;
@@ -180,7 +178,7 @@ public abstract class InsnFormat {
      */
     protected static String regListString(RegisterSpecList list) {
         int sz = list.size();
-        StringBuffer sb = new StringBuffer(sz * 5 + 2);
+        StringBuilder sb = new StringBuilder(sz * 5 + 2);
 
         sb.append('{');
 
@@ -246,7 +244,7 @@ public abstract class InsnFormat {
      * @return {@code non-null;} the string form
      */
     protected static String literalBitsString(CstLiteralBits value) {
-        StringBuffer sb = new StringBuffer(100);
+        StringBuilder sb = new StringBuilder(100);
 
         sb.append('#');
 
@@ -271,7 +269,7 @@ public abstract class InsnFormat {
      */
     protected static String literalBitsComment(CstLiteralBits value,
             int width) {
-        StringBuffer sb = new StringBuffer(20);
+        StringBuilder sb = new StringBuilder(20);
 
         sb.append("#");
 
