@@ -84,7 +84,7 @@ public final class RopTranslator {
      * true if the parameters to this method happen to be in proper order
      * at the end of the frame (as the optimizer emits them)
      */
-    private boolean paramsAreInOrder;
+    private final boolean paramsAreInOrder;
 
     /**
      * Translates a {@link RopMethod}. This may modify the given
@@ -527,6 +527,7 @@ public final class RopTranslator {
         }
 
         /** {@inheritDoc} */
+        @Override
         public void visitPlainInsn(PlainInsn insn) {
             Rop rop = insn.getOpcode();
             if (rop.getOpcode() == RegOps.MARK_LOCAL) {
@@ -704,7 +705,7 @@ public final class RopTranslator {
 
             RegisterSpecList regs = insn.getSources();
             Constant[] constants = new Constant[] {
-                insn.getInvokeMethod(),
+                insn.getPolymorphicMethod(),
                 insn.getCallSiteProto()
                 };
             DalvInsn di = new MultiCstInsn(opcode, pos, regs, constants);
@@ -761,7 +762,27 @@ public final class RopTranslator {
                      */
                     di = new CstInsn(opcode, pos, regs, cst);
                 }
-
+                // (b/120985556) update the following code
+                // move-object vX, vY
+                // instance-of vY, vX, LMyClass;
+                // into
+                // move-object vX, vY
+                // nop
+                // instance-of vY, vX, LMyClass;
+                DalvInsn previousDi = getPrevNonSpecialInsn();
+                if (opcode.getOpcode() == Opcodes.INSTANCE_OF && previousDi != null) {
+                    int prevOpcode = previousDi.getOpcode().getOpcode();
+                    if (prevOpcode == Opcodes.MOVE_OBJECT
+                        || prevOpcode == Opcodes.MOVE_OBJECT_FROM16
+                        || prevOpcode == Opcodes.MOVE_OBJECT_16) {
+                        if (di.getRegisters().size() > 0 && previousDi.getRegisters().size() > 1
+                            && (di.getRegisters().get(0).getReg()
+                                    == previousDi.getRegisters().get(1).getReg())) {
+                            DalvInsn nopDi = new SimpleInsn(Dops.NOP, pos, RegisterSpecList.EMPTY);
+                            addOutput(nopDi);
+                        }
+                    }
+                }
                 addOutput(di);
             }
         }
@@ -794,6 +815,7 @@ public final class RopTranslator {
         }
 
         /** {@inheritDoc} */
+        @Override
         public void visitFillArrayDataInsn(FillArrayDataInsn insn) {
             SourcePosition pos = insn.getPosition();
             Constant cst = insn.getConstant();
@@ -828,6 +850,16 @@ public final class RopTranslator {
             output.add(insn);
         }
 
+        protected DalvInsn getPrevNonSpecialInsn() {
+            for (int i = output.size() - 1; i >= 0; --i) {
+                DalvInsn insn = output.get(i);
+                if (insn.getOpcode().getOpcode() != Opcodes.SPECIAL_FORMAT) {
+                    return insn;
+                }
+            }
+            return null;
+        }
+
         /**
          * Adds to the output suffix.
          *
@@ -845,7 +877,7 @@ public final class RopTranslator {
     private class LocalVariableAwareTranslationVisitor
             extends TranslationVisitor {
         /** {@code non-null;} local variable info */
-        private LocalVariableInfo locals;
+        private final LocalVariableInfo locals;
 
         /**
          * Constructs an instance.
