@@ -32,7 +32,6 @@ import com.android.dex.SizeOf;
 import com.android.dex.TableOfContents;
 import com.android.dex.TypeList;
 import com.android.dx.command.dexer.DxContext;
-
 import java.io.File;
 import java.io.IOException;
 import java.util.*;
@@ -1145,8 +1144,10 @@ public final class DexMerger {
                 encodedArray += contents.encodedArrays.byteCount * 2;
                 // all of the bytes in an annotations section may be uleb/sleb
                 annotation += (int) Math.ceil(contents.annotations.byteCount * 2);
-                // all of the bytes in a debug info section may be uleb/sleb
-                debugInfo += contents.debugInfos.byteCount * 2;
+                // all of the bytes in a debug info section may be uleb/sleb. The additive constant
+                // is a fudge factor observed to be required when merging small
+                // DEX files (b/68483205).
+                debugInfo += contents.debugInfos.byteCount * 2 + 8;
             }
         }
 
